// HistoryMgr.cpp : implementation file
//

#include "stdafx.h"
#include "HistoryMgr.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CHistoryMgr

/* This CHistoryMgr : history managet allows you to insert
a default of 20 items : files and resources , it updates the contents 
whenever a new item is inserted . it allows you to delete an item.
Dont call CStringArray::GetSize() it returns the total size and 
(also Emty entries) , Theres a modified one (GetCount) that counts only non
empty entries.
*/

CHistoryMgr::CHistoryMgr()
{
	SetMaxItems(20);
}

CHistoryMgr::~CHistoryMgr()
{
}


CString CHistoryMgr::strFilePrefix = _T("File://");
CString CHistoryMgr::strResourcePrefix = _T("Res://");

/////////////////////////////////////////////////////////////////////////////
// CHistoryMgr members

//These 2 functions create string representations of files and resources
void CHistoryMgr::CreateFileString(CString &strOut, LPCTSTR lpszPath)
{
	strOut = strFilePrefix;
	strOut += lpszPath;
}
void CHistoryMgr::CreateResourceString(CString &strOut, LPCTSTR lpszResourceName, LPCTSTR lpszResourceType)
{
	strOut = strResourcePrefix;
	strOut += lpszResourceType;
	strOut += _T('.');
	strOut += lpszResourceName;
}
BOOL CHistoryMgr::Add( LPCTSTR lpszPath)
{
	if (!lpszPath)  return FALSE;
	CString strFileName;
	CreateFileString( strFileName, lpszPath);
	return InsertToPlace(strFileName);
}

BOOL CHistoryMgr::Add( LPCTSTR lpszResourceName, LPCTSTR lpszResourceType)
{
	if (!lpszResourceName)  return FALSE;
	if (!lpszResourceType)  return FALSE;

	CString strFileName;
	CreateResourceString( strFileName, lpszResourceName, lpszResourceType);
	return InsertToPlace(strFileName);
}

int CHistoryMgr::Find( LPCTSTR lpszFileName)
{
	for (int iSize = GetCount(), i=0 ; i<iSize ; i++) {
		CString str = GetAt(i);
		if (!str.CompareNoCase(lpszFileName))
			return i;
	}
	return -1;
}
//this version return only non empty entries
int CHistoryMgr::GetCount()
{
	int iSize = GetSize();
	for (int i=0,iCount=0 ; i<iSize ; i++) {
		if (!IsEmpty(i)) //If an entry isnt empty then count it
			iCount++;
	}
	return iCount;	
}

BOOL CHistoryMgr::InsertToPlace( LPCTSTR lpszFileName)
{
	int iFind = Find(lpszFileName);
	if (iFind>=0) //found
		CStringArray::RemoveAt(iFind);

	int iSize = GetSize();
	for (int i= iSize-1 ; i>0 ; i--) {
		CString str = GetAt(i-1);
		if (!str.IsEmpty())
			SetAt( i, str);
	}
	SetAt( 0, lpszFileName);

	RemoveExtra();
	return TRUE;
}

void CHistoryMgr::RemoveAt(int nIndex) //removes an index
{
	ASSERT(IsValid(nIndex));
	if (IsEmpty(nIndex)) return;//empty not needed

	int iSize = GetCount();//size of non empty
	for (int i = nIndex ; i<iSize-1 ; i++) {
		CString str = GetAt(i+1);
		if (str.GetLength())
			SetAt( i, str);
	}
	SetAt(i,_T("")); //empty last item

}
void CHistoryMgr::RemoveAt( LPCTSTR lpszPath) //removes a file name
{
	if (!lpszPath)  return;
	CString strFileName;
	CreateFileString( strFileName, lpszPath);

	int iFind = Find(strFileName);
	if (iFind != -1)
		RemoveAt(iFind);
}
void CHistoryMgr::RemoveAt( LPCTSTR lpszResourceName, LPCTSTR lpszResourceType) //removes a resource
{
	if (!lpszResourceName)  return;
	if (!lpszResourceType)  return;

	CString strFileName;
	CreateResourceString( strFileName, lpszResourceName, lpszResourceType);

	int iFind = Find(strFileName);
	if (iFind != -1)
		RemoveAt(iFind);	
}

//whenever we insert an item and total is more than iMaxItems
//delete the oldest items (located in end of array : we 
//always insert to index 0 .
void CHistoryMgr::RemoveExtra()
{
	while (GetSize() > iMaxItems) 
		CStringArray::RemoveAt( GetSize()-1 , 1);
}

BOOL CHistoryMgr::IsFile(int nIndex)
{
	ASSERT(IsValid(nIndex));
	int iSize = GetSize();
	if ((nIndex>=iSize) && (nIndex<0))
		return FALSE;//index invalid

	CString strFileName = GetAt(nIndex);
	if (strFileName.IsEmpty())
		return FALSE;

	int iFind = strFileName.Find(_T("Res://"));
	if (iFind == -1) //in place 0 : in start!
		return TRUE;
	return FALSE;
}
BOOL CHistoryMgr::IsEmpty(int nIndex)
{
	ASSERT(IsValid(nIndex));
	CString str = GetAt(nIndex);
	return (str.GetLength()==0);
}

BOOL CHistoryMgr::GetFile(int nIndex, CString &strFileName)
{
	ASSERT(IsValid(nIndex));
	if (!IsFile(nIndex))  return FALSE;//not a file

	CString strFull = GetAt(nIndex);
	int iFind = strFull.ReverseFind(_T('/'));
	strFileName = strFull.Mid(iFind+1);
	return TRUE;
}
BOOL CHistoryMgr::GetResource(int nIndex, CString &strResourceName, CString &strResourceType)
{
	ASSERT(IsValid(nIndex));
	if (IsFile(nIndex))  return FALSE;//not a resource
	
	CString strFull = GetAt(nIndex);
	if (!strFull.GetLength())
		return FALSE;
	int iFind = strFull.ReverseFind(_T('.'));
	strResourceName	= strFull.Mid(iFind+1);

	iFind = strFull.ReverseFind(_T('/'));
	strResourceType = strFull.Mid(iFind+1);
	iFind = strResourceType.ReverseFind(_T('.'));
	strResourceType = strResourceType.Left(iFind);

	return TRUE;
}