// MultiLineEditCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "MultiLineEditCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMultiLineEditCtrl

CMultiLineEditCtrl::CMultiLineEditCtrl()
{
}

CMultiLineEditCtrl::~CMultiLineEditCtrl()
{
}

//static : the style
DWORD CMultiLineEditCtrl::dwMultiLineStyle = ES_WANTRETURN|
	ES_READONLY|WS_VSCROLL|WS_HSCROLL|WS_CHILD|WS_VISIBLE|
	ES_AUTOVSCROLL|ES_AUTOHSCROLL|ES_MULTILINE;

BEGIN_MESSAGE_MAP(CMultiLineEditCtrl, CEdit)
	//{{AFX_MSG_MAP(CMultiLineEditCtrl)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMultiLineEditCtrl message handlers

BOOL CMultiLineEditCtrl::PreCreateWindow(CREATESTRUCT& cs) 
{
	cs.style = (LONG)dwMultiLineStyle;
	return CEdit::PreCreateWindow(cs);
}

void CMultiLineEditCtrl::PreSubclassWindow() 
{
	ModifyStyle( 0, dwMultiLineStyle, SWP_NOSIZE|SWP_NOMOVE|SWP_NOZORDER);
	CEdit::PreSubclassWindow();
}

BOOL CMultiLineEditCtrl::SubclassDlgItem( UINT nID, CWnd *pParent ,LPCTSTR lpszText)
{
	if (CEdit::SubclassDlgItem(nID, pParent)) {
		SetMultiLineText(lpszText);
		return TRUE;
	}
	return FALSE;
}

BOOL CMultiLineEditCtrl::ReadFromFile( LPCTSTR lpszFileName)
{
	CStdioFile file;
	CFileException err;
	CString strFilePath = lpszFileName;
	try {
		if (file.Open( strFilePath , CFile::modeRead ,&err)) {
			DWORD cbSize = file.GetLength();
			if (cbSize<60000) {
				CArchive ar( &file, CArchive::load);
				Serialize(ar,cbSize);
				return TRUE;
			}
		}//file opened
	}
	catch (CFileException e) {
		e.ReportError();
	}
	return FALSE;
}

BOOL CMultiLineEditCtrl::SaveToFile(LPCTSTR lpszFileName,BOOL bOverriteExisting)
{
	CStdioFile file;
	CFileException err;
	CString strFilePath = lpszFileName;
	try {
		UINT uFlags = (bOverriteExisting) ? CFile::modeCreate|CFile::modeWrite : CFile::modeWrite;
		
		if (file.Open( strFilePath , uFlags ,&err)) {
			DWORD cbSize = file.GetLength();
			if (cbSize<60000) {
				CArchive ar( &file, CArchive::store);
				Serialize(ar,cbSize);
				return TRUE;
			}
		}//file opened
	}
	catch (CFileException e) {
		e.ReportError();
	}
	return FALSE;
}

void CMultiLineEditCtrl::Serialize(CArchive& ar, UINT nLen)
{
	if (ar.IsLoading()) { //loading
		TCHAR *lpszText = new TCHAR[nLen+1];
		if (!lpszText)  return;
		UINT uRead = ar.Read( lpszText, nLen*sizeof(TCHAR));
	
		// Replace the editing edit buffer with the newly loaded data
		lpszText[uRead] = '\0';
 
		SetMultiLineText(lpszText);
		delete [] lpszText;	//delete the buffer
	}
	else { // saving
		CString string;
		GetMultiLineText(string);
		int iLength = string.GetLength();

		ar.Write( (LPCTSTR)string , iLength*sizeof(TCHAR));
	}
}

// These function are needed in order to convert a stream of
// text to be properly shown or got in multiline edit ctrl

//whenever you get text from a multiline edit ctrl you should
// delete the '\r' character from the series of \r\n (10,13)
BOOL CMultiLineEditCtrl::GetMultiLineText(CString &rString)
{
	GetWindowText(rString);
	for (int i=0 ; i<rString.GetLength() ; i++ ) {
		if ((rString.GetAt(i) == _T('\n') )) {
			if ((i) && (rString.GetAt(i-1) == _T('\r')))
					rString.Delete(i-1,1);
		}
	}
	return TRUE;
}
//whenever you insert a text to edit ctrl each '\n' shoult be
//converted to series \r\n (10,13) (CR and LF) in order that new line
//be presented nicely
BOOL CMultiLineEditCtrl::SetMultiLineText(LPCTSTR lpszText)
{
	CString string = lpszText;
	for (int i=0 ; i<string.GetLength() ; i++ ) {
		if ((string.GetAt(i) == _T('\n') )) {
			if (!i) string.Insert(i,_T('\r'));
			else if (string.GetAt(i-1) != _T('\r'))
					string.Insert(i,_T('\r'));
			i++;
		}
	}
	SetWindowText(string);
	Invalidate();
	return TRUE;
}
CString CMultiLineEditCtrl::GetMultiText()
{
	CString strText;
	GetMultiLineText(strText);
	return strText;
}

BOOL CMultiLineEditCtrl::LoadFromResource( UINT uResourceName)
{
	CString strText;
	if (strText.LoadString(uResourceName)) {
		return 	SetMultiLineText((LPCTSTR)strText);
	}
	return FALSE;
}



