// StaticLink.cpp : implementation file
//

#include "stdafx.h"
#include "StaticLink.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
// Copyright 1998 Paul DiLascia
// If this code works, it was written by Paul DiLascia.
// If not, I don't know who wrote it.
//
// CStaticLink implements a static control that's a hyperlink
// to any file on your desktop or web. You can use it in dialog boxes
// to create hyperlinks to web sites. When clicked, opens the file/URL


/* Extended by Amir Israeli July 2000 
Email: israelaq@walla.co.il
Site: amir-israeli.tripod.com/default.htm
Features added:
a) Undeline font
b) tooltips (option of tracking tooltips)
*/
/////////////////////////////////////////////////////////////////////////////
// CStaticLink

CStaticLink::CStaticLink()
{
	m_color = CStaticLink::g_colorUnvisited;
	SetState(MOUSE_OUT);
	iPrevState = MOUSE_OUT;
	bWindowTipEnabled = TRUE;
}

CStaticLink::~CStaticLink()
{
}

COLORREF CStaticLink::g_colorUnvisited = 0xFF000000|RGB(0,0,255);		 // blue
COLORREF CStaticLink::g_colorVisited   = 0xFF000000|RGB(128,0,128);		 // purple
HCURSOR	CStaticLink::g_hCursorLink = NULL;

CFont CStaticLink::m_font; //regular font
CFont CStaticLink::m_fontU;// underlined font

CWindowTip CStaticLink::m_tooltip;// Extended tool tip
UINT  CStaticLink::uTipID = ID_WINDOW_TIP;//tip id : the programmer must ensure that this 
//id exist or it should initialize it before use (see also dialog::OnInitDialog)


BEGIN_MESSAGE_MAP(CStaticLink, CStatic)
	//{{AFX_MSG_MAP(CStaticLink)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_SETCURSOR()
	ON_WM_CTLCOLOR_REFLECT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
//////
BOOL CStaticLink::PreCreateWindow(CREATESTRUCT& cs) 
{
	cs.style |= (SS_NOTIFY|SS_LEFT);// 
	return CStatic::PreCreateWindow(cs);
}

void CStaticLink::PreSubclassWindow() 
{
	ModifyStyle( 0, SS_NOTIFY|SS_LEFT, 0);// 
	CStatic::PreSubclassWindow();
}

//You have 2 options of setting the tips to this control :
//you can rely on the CWindowTip member that is contained inside (as static)
//instead you can use CToolTipEx object (you see an example of setting tips
//to control in the dialog in "SomeCtrlsDlg.cpp"
BOOL CStaticLink::SubclassDlgItem(UINT nID, CWnd* pParent, LPCTSTR lpszText ,LPCTSTR lpszLink)
{
	if (CStatic::SubclassDlgItem(nID, pParent)) {
		::SetWindowText( m_hWnd, lpszText);
		if (lpszLink)   
			SetLink(lpszLink);
		return TRUE;
	}
	return FALSE;
}

BOOL CStaticLink::CreateToolTip()
{
	if (!bWindowTipEnabled)
		return FALSE;
	if (!m_tooltip.GetSafeHwnd())
		return m_tooltip.Create( AfxGetMainWnd(), TRUE);
	return TRUE;
}

void CStaticLink::OnMouseEnter()
{
	SetState(MOUSE_OVER);
	SetCapture();
	Invalidate();

	CPoint point(::GetMessagePos());
	if (!bWindowTipEnabled)	 return; // dont call/use to tip support
	if (!m_tooltip.GetSafeHwnd())
		m_tooltip.Create( this, uTipID, 2000, TRUE);
	m_tooltip.SetText(m_link);
}
void CStaticLink::OnMouseLeave()
{
	//only if tooltip support is enabled call it
	if ((bWindowTipEnabled) && (m_tooltip.GetSafeHwnd()))
		m_tooltip.EndWindow();

	SetState(MOUSE_OUT);
	ReleaseCapture();

	CClientDC  dc(this);
	SetDCStatus(&dc);
	CString strText;
	GetWindowText(strText);
	CRect rect;
	GetClientRect(&rect);
	dc.DrawText( strText, &rect, DT_LEFT|DT_VCENTER);
}

void CStaticLink::SetDCStatus(CDC *pDC)
{
	if (!pDC)  return;
	if (GetState() == MOUSE_OVER) {
		pDC->SelectObject(&m_fontU);
		pDC->SetBkMode(TRANSPARENT);
	}
	else {
		pDC->SelectObject(&m_font);
		pDC->SetBkMode(OPAQUE);// TRANSPARENT );
	}
	pDC->SetTextColor(m_color);
	pDC->SetBkColor(::GetSysColor(COLOR_3DFACE));
}


HBRUSH CStaticLink::CtlColor(CDC* pDC, CWnd* pWnd) 
{
	DWORD dwStyle = GetStyle();
	HBRUSH hbr = NULL;
	if ((dwStyle & 0xFF) <= SS_RIGHT) {

		// this is a text control: set up font and colors
		if (!m_font.GetSafeHandle()) {
			// first time init: create font
			LOGFONT lf;
			GetFont()->GetObject(sizeof(lf), &lf);
			lf.lfUnderline = FALSE;
			m_font.CreateFontIndirect(&lf);
		}
		if (!m_fontU.GetSafeHandle()) {
			// first time init: create font
			LOGFONT lf;
			GetFont()->GetObject(sizeof(lf), &lf);
			lf.lfUnderline = TRUE;
			m_fontU.CreateFontIndirect(&lf);
		}

		SetDCStatus(pDC);

		// return hollow brush to preserve parent background color
		hbr = (HBRUSH)::GetStockObject(HOLLOW_BRUSH);
	}
	return hbr;
}


BOOL CStaticLink::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	if (g_hCursorLink == NULL) {
		static bTriedOnce = FALSE;
		if (!bTriedOnce) {
			EnableToolTips(TRUE);
			CString windir;
			GetWindowsDirectory(windir.GetBuffer(MAX_PATH), MAX_PATH);
			windir.ReleaseBuffer();
			windir += _T("\\winhlp32.exe");
			HMODULE hModule = LoadLibrary(windir);
			if (hModule) {
				g_hCursorLink =
					CopyCursor(::LoadCursor(hModule, MAKEINTRESOURCE(106)));
			}
			FreeLibrary(hModule);
			bTriedOnce = TRUE;
		}
	}
	if (g_hCursorLink) {
		::SetCursor(g_hCursorLink);
		return TRUE;
	}
	return FALSE;
}

void CStaticLink::OnLButtonDown(UINT nFlags, CPoint point) 
{
	OnMouseLeave();
	if (m_link.IsEmpty()) //empty link
		return;
	Navigate();
}

void CStaticLink::OnMouseMove(UINT nFlags, CPoint point) 
{
	CStatic::OnMouseMove(nFlags, point);

	// 1. Mouse has moved and we are not tracking this button, or 
	// 2. mouse has moved and the cursor was not above this window
	// == Is equivalent to WM_MOUSEENTER (for which there is no message)
	if ((GetCapture()!=this) && (GetState() == MOUSE_OUT))
		OnMouseEnter();
	else {
		if (GetState() == MOUSE_OVER) {
			CRect rc;
			GetClientRect(&rc);
			if (!rc.PtInRect(point)) // The mouse cursor is no longer above this button
				OnMouseLeave();
		}
	}
}

BOOL CStaticLink::Navigate()
{
	HINSTANCE h = (m_link.IsEmpty()) ? NULL :
		ShellExecute(0, _T("open"), m_link, 0, 0, SW_SHOWNORMAL);
	if (h > (HINSTANCE)32) {
		m_color = g_colorVisited;
		return TRUE;
	}
	return FALSE;
}

