// WindowTip.cpp : implementation file
//

#include "stdafx.h"
#include "WindowTip.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////
//////////   CNonClientMetrics
class CNonClientMetrics : public NONCLIENTMETRICS {
public:
	CNonClientMetrics() {
		cbSize = sizeof(NONCLIENTMETRICS);
		SystemParametersInfo(SPI_GETNONCLIENTMETRICS,0,this,0);
	}
};

/////////////////////////////////////////////////////////////////////////////
// CWindowTip
CWindowTip::CWindowTip()
{
	bTrackingMode = FALSE;
	m_szMargins = CSize(4,2);

	CNonClientMetrics ncm;	// create font -- use system tooltip font
	m_font.CreateFontIndirect(&ncm.lfStatusFont);

	m_ScreenSize = CSize (::GetSystemMetrics(SM_CXSCREEN),
						  ::GetSystemMetrics(SM_CYSCREEN));
	uTimerID = 0;
	iTimeOfTip = 2000 ;//2 Seconds interval of showing tip
}

CWindowTip::~CWindowTip()
{
	StopTimer();
}


CSize CWindowTip::szOffset = CSize(15,15); 
// Tooltip position when tracking relative to cursor point (left and bottom)

CString CWindowTip::strStandardTextAddition = _T("\nThis is a data-tip window\nNot a part of CToolTipCtrl");
//standard text to be appended in the end of the text string
//This exist so you can add you name or logo of company on any of tooltips
//you create this way (until you annoy everyone)

UINT  CWindowTip::uTimerInterval = 60;
//interval of timer : how delicate will tracking of tip window will be

BEGIN_MESSAGE_MAP(CWindowTip, CWnd)
	//{{AFX_MSG_MAP(CWindowTip)
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//////////////////
// Register class if needed
//
BOOL CWindowTip::PreCreateWindow(CREATESTRUCT& cs) 
{
	static CString sClassName;
//	if (sClassName.IsEmpty())
		sClassName = AfxRegisterWndClass(0);

	cs.lpszClass = sClassName;
	strClassName = sClassName;
	cs.style |= (WS_BORDER|WS_VISIBLE);//    // |WS_POPUP
	return CWnd::PreCreateWindow(cs);
}

void CWindowTip::OnTimer(UINT nIDEvent) 
{
	if (uTimerID == nIDEvent) {

		if (iElapedTime >= iTimeOfTip) //reaches the end 
			EndWindow(); // only at non tracking state
		
		if (!bTrackingMode) //in non tracking mode we add its timer interval
			iElapedTime += uTimerInterval; //until reach of time limit then window is hidden/destroyed
		else { //tracking mode
			MoveWindow();
		}
	}
	else CWnd::OnTimer(nIDEvent);
}

BOOL CWindowTip::CreateTimer()
{
	iElapedTime = 0;
	if (uTimerID)  return TRUE;

	for (UINT nIDEvent = 1000 ; nIDEvent<1500 ; nIDEvent++) {
		if (SetTimer( nIDEvent, uTimerInterval, NULL)) {
			uTimerID = nIDEvent;
			return TRUE;
		}
	}
	return FALSE;
}
BOOL CWindowTip::StopTimer()
{
	if (uTimerID) {
		KillTimer((int)uTimerID);
		uTimerID = 0;
	}
	return TRUE;
}
///////////////////////////////////////////////////////////

//////////////////
// Paint the text. Use system colors
void CWindowTip::OnPaint()
{
	CPaintDC dc(this);
	CRect rc;
	GetClientRect(&rc);
	CString str = strText;
	CBrush b(GetSysColor(COLOR_INFOBK)); // COLOR_WINDOW use tooltip bg color
	dc.FillRect(&rc, &b);

	// draw text
	dc.SetBkMode(TRANSPARENT);
	CFont* pOldFont = dc.SelectObject(&m_font);
	dc.SetTextColor(GetSysColor(COLOR_INFOTEXT)); // tooltip text color

	dc.DrawText(str, &rc, DT_CENTER|DT_VCENTER);
	dc.SelectObject(pOldFont);
}

void CWindowTip::OnDestroy() 
{
	StopTimer();
	CWnd::OnDestroy();
}
void CWindowTip::OnLButtonDown(UINT nFlags, CPoint point) 
{
	EndWindow();
	CWnd::OnLButtonDown(nFlags, point);
}
////////////////////////////////////////////////////////////
////////

BOOL CWindowTip::Create( CWnd* pParentWnd, UINT nID, int iTimeOfTip , BOOL bTrackingMode)
{
	if (::IsWindow(m_hWnd))  return TRUE;
	this->bTrackingMode = bTrackingMode;
	this->iTimeOfTip = iTimeOfTip;

	BOOL bResult = CreateEx(0,//WS_EX_TOPMOST,// ,  // |WS_EX_TOOLWINDOW,//
			strClassName,NULL,
			WS_CHILD  ,// // ,//  |WS_POPUP
			CRect(CPoint(0,0),CSize(0,0)),
			CWnd::GetDesktopWindow(),nID);///pParentWnd
	return bResult;
}

void CWindowTip::SetText( LPCTSTR lpszText)
{
	if (lpszText) {
		strText = lpszText;
		strText += strStandardTextAddition; //adding a standard text
		StartWindow();	// to show you support of data-like tips
	}
}

void CWindowTip::MoveWindow()
{
	CPaintDC dc(this);
	CFont* pOldFont = dc.SelectObject(&m_font);

	CRect rc;
	dc.DrawText( strText, &rc, DT_CALCRECT|DT_CENTER|DT_VCENTER);
	dc.SelectObject(pOldFont);

	CPoint point;
	::GetCursorPos(&point);
	point += szOffset;

	rc.InflateRect(m_szMargins);
	SetWindowPos( NULL, point.x, point.y, rc.Width(), rc.Height(), SWP_SHOWWINDOW|SWP_SHOWWINDOW);
	CWnd::MoveWindow( point.x, point.y, rc.Width(), rc.Height());

	ShowWindow(SW_SHOW);
	UpdateWindow();
}

BOOL CWindowTip::StartWindow()
{
	CreateTimer();
	MoveWindow();
	return TRUE;
}
BOOL CWindowTip::EndWindow()
{
	if (!::IsWindow(m_hWnd))
		return TRUE;
	if (IsWindowVisible())
		ShowWindow(SW_HIDE);
	StopTimer();
	return TRUE;
}

void CWindowTip::PostNcDestroy() 
{
	delete this; //in case of using 'new' to create such
	//window : CWnd::PostNcDestroy() doesn't include delete this (delete the object)
}
